"""Declare app configuration for Django Use Email as Username."""
from django.apps import AppConfig


class DjangoUseEmailAsUsernameConfig(AppConfig):
    """Define app configuration class."""

    name = "django_use_email_as_username"
